;====================================================================
;         Mattel Aquarius Hardware and System ROM Definitions
;====================================================================
; based on work by Kenny Millar and V D Steenovan
;
;
; 2015-11-4  V0.0 created by Bruce Abbott
; 2015-11-13 V0.1 changed ROWCOUNT to LISTCNT to avoid conflict with
;                 BLBASIC graphics variable ROWCOUNT
;
;-------------------------------------------------------------------
;                            IO Ports
;-------------------------------------------------------------------
;   Address         Write                         Read
;      FC       bit 0 = Speaker & tape out    bit 0 = Tape in
;      FD       bit 0 = CP/M memory map       bit 0 = Vertical sync
;      FE       bit 0 = Printer TXD           bit 0 = Printer CTS
;      FF     bits7-0 = Exp bus lock       bits 0-6 = Keyboard
;


; constants:

LINBUFLEN = 73   ; size of line input buffer


;-------------------------------------------------------------------
;                         Screen RAM
;-------------------------------------------------------------------
; 1k for characters followed by 1k for color attributes.
; 1000 visible characters on screen, leaving 24 unused bytes at end.
; First character in screen also sets the border character and color.
; The first row (40 bytes), and first & last columns of each row are
; normally filled with spaces, giving an effective character matrix
; of 38 columns x 24 rows.

CHRRAM   = $3000 ; 12288           start of character RAM
;          $33E7 ; 13287           end of character RAM
                 ;                 24 unused bytes
COLRAM   = $3400 ; 13312           Start of colour RAM
;          $37E7 ; 14311           end of color RAM
                 ;                 24 unused bytes

;-------------------------------------------------------------------
;                       System Variables
;-------------------------------------------------------------------
;Name    location Decimal alt name          description
CURCOL   = $3800 ; 14336  TTYPOS   Current cursor column
CURRAM   = $3801 ; 14337  CHRPOS   Position in CHARACTER RAM of cursor
USRJMP   = $3803 ; 14339  USRGO    JMP instruction for USR.
USRADDR  = $3804 ; 14340  USRAL    address of USR() function
                 ; 14341  USRAH
UDFADDR  = $3806 ; 14342  HOKDSP   RST $30 vector, hooks into various system routines
                 ; 14343
LISTCNT  = $3808 ; 14344  ROWCOUNT Counter for lines listed (pause every 24 lines)
LASTFF   = $3809 ; 14345  PTOLD    Last protection code sent to port($FF)
LSTASCI  = $380a ; 14346  CHARQ    ASCII value of last key pressed.
KWADDR   = $380b ; 14347  SKEY     Address of keyword in the keyword table.
                 ; 14348
CURHOLD  = $380d ; 14349  BUFO     holds character under the cursor.
LASTKEY  = $380E ; 14350  CTRLFG   SCAN CODE of last key pressed. ???
SCANCNT  = $380f ; 14351  FLAG     number of SCANS key has been down for. ???
FDIV     = $3810 ; 14352  FDIV     subroutine for division ???
                 ;  ...
RANDOM   = $381F ; 14367  RANDOM   used by random number generator
                 ; ...
LPTLST   = $3845 ; 14405  LPTLST   last printer operation status
PRNCOL   = $3846 ; 14406  LPTPOS   The current printer column (0-131).
CHANNEL  = $3847 ; 14407  PRTFLG   Channel: 0=screen, 1=printer.
LINLEN   = $3848 ; 14408  LINLEN   line length (initially set to 40 ???)
CLMLST   = $3849 ; 14409  CLMLST   position of last comma column
RUBSW    = $384A ; 14410  RUBSW    rubout switch
STKTOP   = $384B ; 14411  STKTOP   high address of stack. followed by string storage space
                 ; 14412
CURLIN   = $384D ; 14413  CURLIN   current BASIC line number (-1 in direct mode)
                 ; 14414
BASTART  = $384F ; 14415  TXTTAB   pointer to start of BASIC program
                 ; 14416
CASNAM   = $3851 ; 14417  CASNAM   tape filename (6 chars)
CASNM2   = $3857 ; 14423  CASNM2   tape read filename (6 chars)
CASFL2   = $385D ; 14429  CASFL2   tape flag ???
CASFL3   = $385E ; 14430  CASFL3   tape flag (break key check)
BUFMIN   = $385F ; 14431  BUFMIN   buffer used by INPUT statement
LINBUF   = $3860 ; 14432  BUF      line input buffer (73 bytes).
                 ;  ...
BUFEND   = $38A9 ; 14505  BUFEND   end of line unput buffer
DIMFLG   = $38AA ; 14506  DIMFLG   flag variable type
VALTYP   = $38AB ; 14507  VALTYP   flag string/numeric
DORES    = $38AC ; 14508  DORES    flag for crunch
RAMTOP   = $38AD ; 14509  MEMSIZ   Address of top of physical RAM.
                 ; 14510
STRBUF   = $38AF ; 14511           18 bytes used by string functions
                 ;  ...
FRETOP   = $38C1 ; 14529  FRETOP   pointer to top of string space
                 ; 14530
SYSTEMP  = $38C3 ; 14531  TEMP     temp space used by FOR etc.
                 ;  ...
TMPSTAT  = $38ce ; 14540           temp holder of next statement address
                 ;  ...
CONTLIN  = $38d2 ; 14546           Line number to CONTinue from.
CONTPOS  = $38d4 ; 14548           address of line to CONTinue from.
BASEND   = $38d6 ; 14550  VARTAB   variable table (end of BASIC program)
                 ; 14551
ARYTAB   = $38D8 ; 14552  ARYTAB   start of array table
                 ; 14553
ARYEND   = $38DA ; 14554           end of array table
                 ; 14555
RESTORE  = $38DC ; 14556           Address of line last RESTORE'd
                 ; 14557
;          $38DE ; 14558           pointer and flag for arrays
                 ;  ...
FACLO    = $38E4 ; 14564  FACLO    floating point number
                 ;  ...
;          $38F9 ; 14585           used by keybord routine
                 ;  ...
; end of system variables = start of BASIC program space in stock Aquarius
;          $3901 ; 14593


; system routines

; system routines

; RST $08,xx CHKNXT    syntax error if char at (HL) is not eqaul to xx
; RST $10    GETNXT    get char at (HL)+, Carry set if '0' to '9'
; RST $18    PRNTCHR   print char in A
; RST $20    CMPHLDE   compare HL to DE. Z if equal, C if DE is greater
; RST $28    TSTSIGN   test sign of floating point number
; RST $30,xx CALLUDF   hooks into various places in the ROM (identified by xx)
; RST $38    CALLUSR   maskable interrupt handler
; RST $66       -      NMI entry point. No code in ROM for this, do NOT use it!

PRNCHR      = $1d94  ; print character in A
PRNCHR1     = $1d72  ; print character with pause at end of page
PRINTSTR    = $0e9d  ; print null-terminated string
PRNCRLF     = $19ea  ; print CR+LF
CLRKEYWT    = $19da  ; flush keyboard buffer and wait for keypress
SVCURCOL    = $1e3e  ; save cursor position HL = address, A = column
LINEDONE    = $19e5  ; pressed CR during line entry
SCROLLUP    = $1dfe  ; scroll the screen up 1 line
STRTOVAL    = $069c  ; DE = value of decimal number string at (HL)
FINDLIN     = $049f  ; find address of BASIC line DE = line number
PRINTINT    = $1675  ; print integer number HL = 16 bit integer
PRNCHR1     = $1d72  ; print character of listing (pause at end of page)
ERROR_SN    = $03c4  ; show syntax error and return to immediate mode
ERROR_UL    = $06f3  ; line does not exist


