;******************************************************************************
;                      CH376 Play Raw Wave File
;******************************************************************************

 include "aquarius.i"

; I/O ports
CH376_DATA_PORT         equ     $E8     ; change this to match your hardware!
CH376_CONTROL_PORT      equ     CH376_DATA_PORT+1 ; A0 = high
AY_CTRL                 equ     $f7
AY_DATA                 equ     $f6


; commands
CH376_CMD_CHECK_EXIST   equ     $06     ; check if file exists
CH376_CMD_SET_FILE_SIZE equ     $0D     ; set file size
CH376_CMD_SET_USB_MODE  equ     $15     ; set USB mode
CH376_CMD_GET_STATUS    equ     $22     ; get status
CH376_CMD_RD_USB_DATA   equ     $27     ; read data from USB
CH376_CMD_WR_REQ_DATA   equ     $2D     ; write data to USB
CH376_CMD_SET_FILE_NAME equ     $2F     ; set name of file to open, read etc.
CH376_CMD_DISK_MOUNT    equ     $31     ; mount disk
CH376_CMD_FILE_OPEN     equ     $32     ; open file
CH376_CMD_FILE_ENUM_GO  equ     $33     ; get next file info
CH376_CMD_FILE_CREATE   equ     $34     ; create new file
CH376_CMD_FILE_CLOSE    equ     $36     ; close opened file
CH376_CMD_BYTE_READ     equ     $3A     ; start reading bytes
CH376_CMD_BYTE_RD_GO    equ     $3B     ; continue reading bytes
CH376_CMD_BYTE_WRITE    equ     $3C     ; start writing bytes
CH376_CMD_BYTE_WR_GO    equ     $3D     ; continue writing bytes
; status codes
CH376_INT_SUCCESS       equ     $14     ; command executed OK
CH376_INT_DISK_READ     equ     $1D     ; read again
CH376_ERR_INT_DISK_WRT  equ     $1E     ; write error
CH376_ERR_OPEN_DIR      equ     $41     ; is directory, not file
CH376_ERR_MISS_FILE     equ     $42     ; file not found

buffer = $5000
buffer_page = $50

   org  $4000

   ld   a,7
   out  (AY_CTRL),a
   ld   a,$ff
   out  (AY_DATA),a
   ld   a,14
   out  (AY_CTRL),a
   ld   a,128
   out  (AY_DATA),a

   call usb_ready
   ret  nz

   ld   hl,filename
   call usb_open_read
   ret  nz
_loop:
   call usb_read_bytes
   ret


filename:
   db   "/RIDERS.AFS",0  ; "/100HZ.AFS",0 ;


; 6 mcycles
PLAYSAMPLE macro
        ld      a,(de)
        out     (AY_DATA),a
        inc     e
        endm

;------------------------------------------------------------------------------
;               Read Bytes from Open File into RAM
;------------------------------------------------------------------------------
usb_read_bytes:
        ld      de,buffer               ; must be on 256 byte boundary
        ld      l,e
        ld      h,0                     ; H _not_ set to buffer page
        LD      c,CH376_DATA_PORT       ; c = CH376 data port
usb_read_more:
        ld      a,CH376_CMD_BYTE_READ
        out     (CH376_CONTROL_PORT),A  ; command: read bytes
        ld      a,$fe
        out     (c),a
        inc     a
        out     (c),a                   ; send number of bytes to read = 65534
_read_loop:
        ld      b,7
        jr      _wait_int_delay
; wait for CH376 interrupt
_wait_int_loop:
        ld      b,11
_wait_int_delay:
        djnz    _wait_int_delay         ; delay to maintain correct playing speed
        ld      a,h
        cp      l                       ; H = buffer page (buffer valid)?
        jr      z,_read_status
        PLAYSAMPLE                      ; yes, play next sample
_read_status:
        in      a,(CH376_CONTROL_PORT)  ; command: read status register
        rla                             ; interrupt flag bit set? (active low)
        jr      c,_wait_int_loop
        ld      h,d                     ; H = buffer page (buffer is now valid)
_get_status:
        ld      a,CH376_CMD_GET_STATUS
        out     (CH376_CONTROL_PORT),a  ; command: get status
        in      a,(c)                   ; read status byte (clears CH376 interrupt flag)

        ld      i,a                     ; i = status

        ld      a,CH376_CMD_RD_USB_DATA
        out     (CH376_CONTROL_PORT),A  ; command: read USB data
        in      b,(c)                   ; B = number of bytes in this block
        jr      z,_read_next_block      ; if 0 bytes then no block to read

_read_byte:
        ld      a,e
        cp      l                       ; if buffer full then just play another sample
        jr      z,_buffer_full
        ini                             ; read byte into buffer
        ld      h,d                     ; keep HL on same page as DE
        jr      nz,_read_byte2          ; if not end of block then read another byte
        ld      b,3
        jr      _read_next_block        ; end of block,
_buffer_full:
        nop
        nop
        nop
        nop                             ; delay to maintain correct playing speed
        nop
        nop
        nop
_read_byte2
        ld      a,7
_delay2:
        dec     a
        jr      nz,_delay2
        nop
        nop
        PLAYSAMPLE
        ini                             ; read next byte into buffer
        ld      h,d                     ; keep HL on same page as DE
        jr      nz,_read_byte
        ld      b,8
_read_next_block:
        djnz    _read_next_block        ; delay to maintain correct playing speed
        PLAYSAMPLE
        ld      a,i
        cp      CH376_INT_SUCCESS
        jr      z,usb_read_more
        cp      CH376_INT_DISK_READ
        ret     nz
        ld      a,CH376_CMD_BYTE_RD_GO
        out     (CH376_CONTROL_PORT),A  ; command: read more bytes
        jr      _read_loop              ; loop back to read next block
usb_read_end:
        RET


;------------------------------------------------------------------------------
;                      Test if File Exists
;------------------------------------------------------------------------------
; Input:  (HL) = filename
;
; Output:    Z = file exists
;           NZ = file not exist, A = error code
;
usb_file_exist:
        PUSH    HL
        CALL    usb_open_read           ; try to open file
        POP     HL
        CP      CH376_ERR_MISS_FILE     ; file missing?
        JR      usb_file_not_exist
        CP      CH376_ERR_OPEN_DIR      ; directory?
        JR      Z,usb_is_dir
        CP      CH376_INT_SUCCESS       ; opened successfully?
        RET     Z
usb_file_not_exist:
usb_is_dir:
        OR      A
        RET


;------------------------------------------------------------------------------
;                      Open a File or Directory
;------------------------------------------------------------------------------
; Input:  HL = filename (null-terminated)
;
; Output: HL = end of filename + 1
;          Z = OK
;          NZ = fail, A = error code
;                         $1D (INT_DISK_READ) too many subdirectories
;                         $41 (ERR_OPEN_DIR) 'filename'is a directory
;                         $42 (CH376_ERR_MISS_FILE) file not found
;
usb_open_read:
        LD      A,CH376_CMD_SET_FILE_NAME
        OUT     (CH376_CONTROL_PORT), A ; command: set file name
usb_op_name:
        LD      A,(HL)
        INC     HL
        OUT     (CH376_DATA_PORT),A     ; send filename to CH376
        OR      A
        JR      NZ,usb_op_name
        LD      A,CH376_CMD_FILE_OPEN
        OUT     (CH376_CONTROL_PORT),A  ; command: open file
; falls into...

;------------------------------------------------------------------------------
;                   Wait for Interrupt and Read Status
;------------------------------------------------------------------------------
; output:  Z = success
;         NZ = fail, A = error code
;
usb_wait_interrupt:
        PUSH    BC
        LD      BC,0                    ; wait counter = 65536
usb_wait_int_loop:
        IN      A,(CH376_CONTROL_PORT)  ; command: read status register
        RLA                             ; int bit set?
        JR      NC,usb_wait_int_end     ; yes,
        DEC     BC                      ; no, counter-1
        LD      A,B
        OR      C
        JR      NZ,usb_wait_int_loop    ; loop until timeout
usb_wait_int_end:
        LD      A,CH376_CMD_GET_STATUS
        OUT     (CH376_CONTROL_PORT),A  ; command: get status
        IN      A,(CH376_DATA_PORT)     ; read status byte
        CP      CH376_INT_SUCCESS       ; test return code
        POP     BC
        RET

;------------------------------------------------------------------------------
;           Test whether CH376 is Present and has USB Disk Inserted
;------------------------------------------------------------------------------
; If disk present then mount it.
;
; Output:  Z = OK
;         NZ = error
;
usb_ready:
        PUSH    BC
        LD      B,10                    ; retry count
usb_rdy_tst:
        LD      A,CH376_CMD_CHECK_EXIST
        OUT     (CH376_CONTROL_PORT),A  ; command: check if CH376 exists
        LD      A,$1A
        OUT     (CH376_DATA_PORT),A     ; send test byte
        IN      A,(CH376_DATA_PORT)
        CP      $E5                     ; byte inverted?
        JR      Z,usb_rdy_m             ; yes, found CH376
        DJNZ    usb_rdy_tst             ; no, try again
        JR      usb_rdy_fail            ; fail NZ
usb_rdy_m:
        CALL    usb_mount               ; mount disk
        JR      z,usb_rdy_ok            ; return OK if mounted
        LD      B,255                   ; retry count
usb_rdy_L1:
        LD      A,CH376_CMD_SET_USB_MODE
        OUT     (CH376_CONTROL_PORT),A  ; command: set USB mode
        LD      A,6
        OUT     (CH376_DATA_PORT),A     ; mode 6
        EX      (SP),HL
        EX      (SP),HL                 ; delay ~20us
        EX      (SP),HL
        EX      (SP),HL
        IN      A,(CH376_DATA_PORT)
        CP      $51                     ; status = $51?
        JR      Z,usb_rdy_mnt           ; yes, mount disk
        DJNZ    usb_rdy_L1              ; no, try again
        JR      usb_rdy_fail            ; fail NZ
usb_rdy_mnt:
        LD      B,255                   ; retry count
usb_rdy_L2:
        CALL    usb_mount               ; try to mount disk
        JR      Z,usb_rdy_ok            ; return OK if mounted
        DJNZ    usb_rdy_L2              ; no, try again
usb_rdy_fail:                           ; fail NZ
usb_rdy_ok:
        POP     BC
        RET

;------------------------------------------------------------------------------
;                            Mount USB Disk
;------------------------------------------------------------------------------
; output:  Z = mounted
;         NZ = not mounted
;
usb_mount:
        LD      A,CH376_CMD_DISK_MOUNT
        OUT     (CH376_CONTROL_PORT),A  ; command: mount disk
        JP      usb_wait_interrupt      ; wait until done



